@@ -10,7 +10,7 @@ import (
 	"fmt"
 
 	"github.com/peterbourgon/ff/v3/ffcli"
-	"tailscale.com/client/tailscale"
+	"tailscale.com/client/local"
 )
 
 var bugReportCmd = &ffcli.Command{
@@ -40,7 +40,7 @@ func runBugReport(ctx context.Context, args []string) error {
 	default:
 		return errors.New("unknown arguments")
 	}
-	opts := tailscale.BugReportOpts{
+	opts := local.BugReportOpts{
 		Note:     note,
 		Diagnose: bugReportArgs.diagnose,
 	}
