@@ -8,7 +8,7 @@
 package tools
 
 import (
-	_ "fybrik.io/crdoc"
+	_ "github.com/elastic/crd-ref-docs"
 	_ "github.com/tailscale/mkctr"
 	_ "honnef.co/go/tools/cmd/staticcheck"
 	_ "sigs.k8s.io/controller-tools/cmd/controller-gen"
