@@ -1,6 +1,5 @@
-// Copyright (c) 2022 Tailscale Inc & AUTHORS All rights reserved.
-// Use of this source code is governed by a BSD-style
-// license that can be found in the LICENSE file.
+// Copyright (c) Tailscale Inc & AUTHORS
+// SPDX-License-Identifier: BSD-3-Clause
 
 package version_test
 
@@ -10,6 +9,7 @@ import (
 	"testing"
 
 	ts "tailscale.com"
+	"tailscale.com/version"
 )
 
 func TestAlpineTag(t *testing.T) {
@@ -40,3 +40,12 @@ func readAlpineTag(t *testing.T, file string) string {
 	}
 	return ""
 }
+
+func TestShortAllocs(t *testing.T) {
+	allocs := int(testing.AllocsPerRun(10000, func() {
+		_ = version.Short()
+	}))
+	if allocs > 0 {
+		t.Errorf("allocs = %v; want 0", allocs)
+	}
+}
