@@ -5,6 +5,8 @@
 
 package magicsock
 
+import "tailscale.com/disco"
+
 func (c *Conn) DontFragSetting() (bool, error) {
 	return false, nil
 }
@@ -19,3 +21,7 @@ func (c *Conn) PeerMTUEnabled() bool {
 
 func (c *Conn) UpdatePMTUD() {
 }
+
+func pmtuShouldLogDiscoTxErr(m disco.Message, err error) bool {
+	return true
+}
