@@ -11,24 +11,15 @@ import (
 	"tailscale.com/types/preftype"
 )
 
-//lint:ignore U1000 used in Windows/Linux tests only
-func mustCIDRs(ss ...string) []netip.Prefix {
-	var ret []netip.Prefix
-	for _, s := range ss {
-		ret = append(ret, netip.MustParsePrefix(s))
-	}
-	return ret
-}
-
 func TestConfigEqual(t *testing.T) {
 	testedFields := []string{
 		"LocalAddrs", "Routes", "LocalRoutes", "NewMTU",
-		"SubnetRoutes", "SNATSubnetRoutes", "NetfilterMode",
-		"NetfilterKind",
+		"SubnetRoutes", "SNATSubnetRoutes", "StatefulFiltering",
+		"NetfilterMode", "NetfilterKind",
 	}
 	configType := reflect.TypeFor[Config]()
 	configFields := []string{}
-	for i := 0; i < configType.NumField(); i++ {
+	for i := range configType.NumField() {
 		configFields = append(configFields, configType.Field(i).Name)
 	}
 	if !reflect.DeepEqual(configFields, testedFields) {
@@ -125,6 +116,16 @@ func TestConfigEqual(t *testing.T) {
 			&Config{SNATSubnetRoutes: false},
 			true,
 		},
+		{
+			&Config{StatefulFiltering: false},
+			&Config{StatefulFiltering: true},
+			false,
+		},
+		{
+			&Config{StatefulFiltering: false},
+			&Config{StatefulFiltering: false},
+			true,
+		},
 
 		{
 			&Config{NetfilterMode: preftype.NetfilterOff},
